// ________________________________________________________________________________________________________
// Copyright (c) 2020 GebraBit Inc. All rights reserved.
//
// This software, related documentation and any modifications thereto (collectively �Software�) is subject
// to GebraBit and its licensors' intellectual property rights under U.S. and international copyright
// and other intellectual property rights laws. 
//
// GebraBit and its licensors retain all intellectual property and proprietary rights in and to the Software
// and any use, reproduction, disclosure or distribution of the Software without an express license agreement
// from GebraBit is strictly prohibited.
 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT IN  
// NO EVENT SHALL GebraBit BE LIABLE FOR ANY DIRECT, SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, 
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
// NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
// OF THE SOFTWARE.
// @Author       	: Sepehr Azimi
// inspired by ADS1292R ProtoCentral source code
// ________________________________________________________________________________________________________

#include "Arduino.h"
#include "GebraBit_ADS1191-2.h"
#include <SPI.h>

int j,i;

volatile byte SPI_RX_Buff[15] ;
volatile static int SPI_RX_Buff_Count = 0;
volatile char *SPI_RX_Buff_Ptr;
volatile bool ads119xdataReceived = false;

uint16_t uecgtemp = 0;
uint16_t resultTemp = 0;
uint16_t secgtemp = 0;
uint16_t statusByte=0;

uint8_t LeadStatus=0;

boolean ads119x::getads119xEcgSamples(const int dataReady,const int chipSelect,ads119xOutputValues *ecgRespirationValues)
{

  if ((digitalRead(dataReady)) == LOW)      // Sampling rate is set to 125SPS ,DRDY ticks for every 8ms
  {
    SPI_RX_Buff_Ptr = ads119xReadData(chipSelect); // Read the data,point the data to a pointer

    for (int i = 0; i < 11; i++)
    {
      SPI_RX_Buff[SPI_RX_Buff_Count++] = *(SPI_RX_Buff_Ptr + i);  // store the result data in array
    }

    ads119xdataReceived = true;
    j = 0;

    for (i = 3; i < 11; i += 2)         
    {
      uecgtemp = (uint16_t) (  ((uint16_t)SPI_RX_Buff[i + 0] << 8) | ( (uint16_t) SPI_RX_Buff[i + 1]));
      secgtemp = (int16_t) (uecgtemp);

      (ecgRespirationValues->sDaqVals)[j++] = secgtemp;  
    }
    ecgRespirationValues->sresultTempResp = (int16_t)(resultTemp);
    ecgRespirationValues->leadoffDetected  = false;
    ads119xdataReceived = false;
    SPI_RX_Buff_Count = 0;
    return true;
  }

  else
    return false;
}

char* ads119x::ads119xReadData(const int chipSelect)
{
  static char SPI_Dummy_Buff[12];
  digitalWrite(chipSelect, LOW);

  for (int i = 0; i < 11; ++i)
  {
    SPI_Dummy_Buff[i] = SPI.transfer(CONFIG_SPI_MASTER_DUMMY);
  }

  digitalWrite(chipSelect, HIGH);
  return SPI_Dummy_Buff;
}

void ads119x::ads119xInit(const int chipSelect,const int pwdnPin,const int startPin)
{
  // start the SPI library:
  ads119xReset(pwdnPin);
  delay(100);
  ads119xDisableStart(startPin);
  ads119xEnableStart(startPin);
  ads119xHardStop(startPin);
  ads119xStartDataConvCommand(chipSelect);
  ads119xSoftStop(chipSelect);
  delay(50);
  ads119xStopReadDataContinuous(chipSelect);					// SDATAC command
  delay(300);
  ads119xRegWrite(ADS119X_REG_CONFIG1, 0x00,chipSelect); 		//Set sampling rate to 125 SPS
  delay(10);
  ads119xRegWrite(ADS119X_REG_CONFIG2, 0b10100000,chipSelect);	//Lead-off comp off, test signal disabled
  delay(10);
  ads119xRegWrite(ADS119X_REG_LOFF, 0b00010000,chipSelect);		//Lead-off defaults
  delay(10);
  ads119xRegWrite(ADS119X_REG_CH1SET, 0b01000000,chipSelect);	//Ch 1 enabled, gain 6, connected to electrode in
  delay(10);
  ads119xRegWrite(ADS119X_REG_CH2SET, 0b01100000,chipSelect);	//Ch 2 enabled, gain 6, connected to electrode in
  delay(10);
  ads119xRegWrite(ADS119X_REG_RLDSENS, 0b00101100,chipSelect);	//RLD settings: fmod/16, RLD enabled, RLD inputs from Ch2 only
  delay(10);
  ads119xRegWrite(ADS119X_REG_LOFFSENS, 0x00,chipSelect);		//LOFF settings: all disabled
  delay(10);													//Skip register 8, LOFF Settings default
  ads119xRegWrite(ADS119X_REG_MISC1, 0b11110010,chipSelect);		
  delay(10);
  ads119xRegWrite(ADS119X_REG_MISC2, 0b00000011,chipSelect);		
  delay(10);
  ads119xStartReadDataContinuous(chipSelect);
  delay(10);
  ads119xEnableStart(startPin);
}

void ads119x::ads119xReset(const int pwdnPin)
{
  digitalWrite(pwdnPin, HIGH);
  delay(100);					// Wait 100 mSec
  digitalWrite(pwdnPin, LOW);
  delay(100);
  digitalWrite(pwdnPin, HIGH);
  delay(100);
}

void ads119x::ads119xDisableStart(const int startPin)
{
  digitalWrite(startPin, LOW);
  delay(20);
}

void ads119x::ads119xEnableStart(const int startPin)
{
  digitalWrite(startPin, HIGH);
  delay(20);
}

void ads119x::ads119xHardStop (const int startPin)
{
  digitalWrite(startPin, LOW);
  delay(100);
}

void ads119x::ads119xStartDataConvCommand (const int chipSelect)
{
  ads119xSPICommandData(START,chipSelect);					// Send 0x08 to the ADS1x9x
}

void ads119x::ads119xSoftStop (const int chipSelect)
{
  ads119xSPICommandData(STOP,chipSelect);                   // Send 0x0A to the ADS1x9x
}

void ads119x::ads119xStartReadDataContinuous (const int chipSelect)
{
  ads119xSPICommandData(RDATAC,chipSelect);					// Send 0x10 to the ADS1x9x
}

void ads119x::ads119xStopReadDataContinuous (const int chipSelect)
{
  ads119xSPICommandData(SDATAC,chipSelect);					// Send 0x11 to the ADS1x9x
}

void ads119x::ads119xSPICommandData(unsigned char dataIn,const int chipSelect)
{
  byte data[1];
  //data[0] = dataIn;
  digitalWrite(chipSelect, LOW);
  delay(2);
  digitalWrite(chipSelect, HIGH);
  delay(2);
  digitalWrite(chipSelect, LOW);
  delay(2);
  SPI.transfer(dataIn);
  delay(2);
  digitalWrite(chipSelect, HIGH);
}

//Sends a write command to SCP1000
void ads119x::ads119xRegWrite (unsigned char READ_WRITE_ADDRESS, unsigned char DATA,const int chipSelect)
{

  switch (READ_WRITE_ADDRESS)
  {
    case 1:
            DATA = DATA & 0x87;
	          break;
    case 2:
            DATA = DATA & 0xFB;
	          DATA |= 0x80;
	          break;
    case 3:
      	    DATA = DATA & 0xFD;
      	    DATA |= 0x10;
      	    break;
    case 7:
      	    DATA = DATA & 0x3F;
      	    break;
    case 8:
    	      DATA = DATA & 0x5F;
	          break;
    case 9:
      	    DATA |= 0x02;
      	    break;
    case 10:
      	    DATA = DATA & 0x87;
      	    DATA |= 0x01;
      	    break;
    case 11:
      	    DATA = DATA & 0x0F;
      	    break;
    default:
            break;
  }
  // now combine the register address and the command into one byte:
  byte dataToSend = READ_WRITE_ADDRESS | WREG;
  digitalWrite(chipSelect, LOW);
  delay(2);
  digitalWrite(chipSelect, HIGH);
  delay(2);
  // take the chip select low to select the device:
  digitalWrite(chipSelect, LOW);
  delay(2);
  SPI.transfer(dataToSend); //Send register location
  SPI.transfer(0x00);		//number of register to wr
  SPI.transfer(DATA);		//Send value to record into register
  delay(2);
  // take the chip select high to de-select:
  digitalWrite(chipSelect, HIGH);
}
